@@ -1,15 +1,34 @@
 package Ado::Plugin::Auth;
 use Mojo::Base 'Ado::Plugin';
+use Mojo::Util qw(class_to_path);
 
 sub register {
     my ($self, $app, $conf) = shift->initialise(@_);
 
     # Make sure we have all we need from config files.
-    $conf->{auth_methods} ||= ['ado', 'facebook'];
+    $conf->{auth_methods} ||= ['ado'];
+    $app->helper(login_ado => \&_login_ado);
+
     $app->config(auth_methods => $conf->{auth_methods});
+    $app->config(ref($self)   => $conf);
+
+
+    #OAuth2 providers
+    my @auth_methods = @{$conf->{auth_methods}}[1 .. $#{$conf->{auth_methods}}];
+
+    if (@auth_methods) {
+        for my $m (@auth_methods) {
+            Carp::croak("Configuration options for authentication method \"$m\" "
+                  . "are not enough!. Please add them.")
+              if (keys %{$conf->{providers}{$m}} < 2);
+        }
+        $app->plugin('OAuth2', $conf->{providers});
+    }
 
     # Add helpers
-    $app->helper(login_ado => \&_login_ado);
+    #oauth2 links - helpers after 'ado'
+    $app->helper(login_google => \&_login_google)
+      if (List::Util::first { $_ eq 'google' } @auth_methods);
 
     # Add conditions
     $app->routes->add_condition(authenticated => \&authenticated);
@@ -20,8 +39,16 @@ sub register {
             return $_[1]->user->ingroup($_[-1]);
         }
     );
+    $app->hook(
+        after_user_add => sub {
+            my ($c, $user, $raw_data) = @_;
+            $app->log->info($user->description . ' $user->id ' . $user->id . ' added!');
+            $c->debug('new user created with arguments:' . $c->dumper($user->data, $raw_data))
+              if $Ado::Control::DEV_MODE;
+        }
+    );
 
-    #Add to classes used for finding templates in DATA sections
+    #Add this package to classes searched for templates in DATA sections
     push @{$app->renderer->classes}, __PACKAGE__;
     return $self;
 }
@@ -62,17 +89,19 @@ sub login {
         $c->session('over_route' => $referrer);
         $c->debug('over_route is ' . $referrer) if $Ado::Control::DEV_MODE;
     }
-    return $c->render(status => 200, template => 'login') if $c->req->method ne 'POST';
+    my $auth_method = Mojo::Util::trim($c->param('auth_method'));
+
+    return $c->render(status => 200, template => 'login')
+      if $c->req->method ne 'POST' && $auth_method eq 'ado';
 
     #derive a helper name for login the user
-    my $auth_method  = Mojo::Util::trim($c->param('auth_method'));
     my $login_helper = 'login_' . $auth_method;
     my $authnticated = 0;
     if (eval { $authnticated = $c->$login_helper(); 1 }) {
         if ($authnticated) {
 
             # Store a friendly message for the next page in flash
-            $c->flash(login_message => 'Thanks for logging in! Wellcome!');
+            $c->flash(login_message => 'Thanks for logging in! Welcome!');
 
             # Redirect to referrer page with a 302 response
             $c->debug('redirecting to ' . $c->session('over_route'))
@@ -141,6 +170,114 @@ sub _login_ado {
     return '';
 }
 
+#used as helper within login()
+# this method is called as return_url after the user
+# agrees or denies access for the application
+sub _login_google {
+    my ($c) = @_;
+    state $app = $c->app;
+    my $provider  = $c->param('auth_method');
+    my $providers = $app->config('Ado::Plugin::Auth')->{providers};
+    $providers->{$provider}{redirect_uri} = '' . $c->url_for("/")->to_abs;
+
+    #second call should get the token it self
+    my $access_token = $c->get_token($provider, %{$providers->{$provider}});
+    $c->debug("in _login_google \$acc_token: " . ($access_token || 'no'));
+    $c->debug("in _login_google error from provider: " . ($c->param('error') || 'no error'));
+
+    #When we have an 'access_denied' from provider we get a reference to $c and not a false value!
+    if ($access_token && !ref($access_token)) {    #Athenticate, create and login the user.
+        return _create_or_authenticate_google_user($c, $access_token, $providers->{$provider});
+    }
+    else {
+        #Redirect to front-page and say sorry
+        # We are very sorry but we need to know you are a reasonable human being.
+        $c->flash(error_login => $c->l('oauth2_sorry') . ($c->param('error') || ''));
+        $c->debug('no token sorry');
+        $c->res->code(307);    #307 Temporary Redirect
+        $c->redirect_to('/');
+    }
+    return;
+}
+
+sub _authenticate_oauth2_user {
+    my ($c, $user, $time) = @_;
+    if (   $user->disabled
+        || ($user->stop_date != 0 && $user->stop_date < $time)
+        || $user->start_date > $time)
+    {
+        $c->flash(login_message => $c->l('oauth2_disabled'));
+        $c->redirect_to('/');
+        return;
+    }
+    $c->session(login_name => $user->login_name);
+    $c->user($user);
+    $c->app->log->info('$user ' . $user->login_name . ' logged in!');
+    return 1;
+}
+
+sub _create_or_authenticate_google_user {
+    my ($c, $access_token, $provider) = @_;
+
+    #make request for the user info
+    my $token_type = 'Bearer';
+    my $ua         = Mojo::UserAgent->new;
+    my $user_info =
+      $ua->get($provider->{info_url} => {Authorization => "$token_type $access_token"})
+      ->res->json;
+    my $U = 'Ado::Model::Users';
+    state $sql = $U->SQL('SELECT') . ' WHERE email=?';
+    my $user = $U->query($sql, $user_info->{email});
+    my $time = time;
+
+    if ($user->id) {
+        return _authenticate_oauth2_user($c, $user, $time);
+    }
+    else {
+        #create the user
+        my %args = ();
+        $args{email}      = $user_info->{email};
+        $args{login_name} = $time . $user_info->{email};
+        $args{login_name} =~ s/\@.+//;
+        $args{login_password} =
+          Mojo::Util::sha1_hex($args{login_name} . Ado::Sessions->generate_id());
+        $args{first_name}  = $user_info->{given_name};
+        $args{last_name}   = $user_info->{family_name};
+        $args{description} = "Registered via $provider->{info_url}!";
+        $args{created_by}  = $args{changed_by} = 1;
+        $args{start_date}  = $args{disabled} = $args{stop_date} = 0;
+        my $app = $c->app;
+
+        if ($user = $U->add(%args)) {
+            $app->plugins->emit_hook(after_user_add => $c, $user, $user_info);
+            $c->user($user);
+            $c->session(login_name => $user->login_name);
+            $app->log->info($user->description . ' $user ' . $user->login_name . ' logged in!');
+            $c->flash(login_message => $c->l('oauth2_wellcome'));
+            $c->redirect_to('/');
+            return 1;
+        }
+        else {
+            $app->log->error($@);
+            return;
+        }
+    }
+    return;
+}
+
+
+# Redirects to Consent screen
+sub authorize {
+    my ($c)    = @_;
+    my $m      = $c->param('auth_method');
+    my $params = $c->app->config('Ado::Plugin::Auth')->{providers}{$m};
+    $params->{redirect_uri} = '' . $c->url_for("/login/$m")->to_abs;
+
+    #This call will redirect the user to the provider Consent screen.
+    $c->redirect_to($c->get_authorize_url($m, %$params));
+    return;
+}
+
 1;
 
 
@@ -150,30 +287,34 @@ sub _login_ado {
 
 =head1 NAME
 
-Ado::Plugin::Auth - Authenticate users
+Ado::Plugin::Auth - Passwordless user authentication for Ado
 
 =head1 SYNOPSIS
 
-  #in ado.${\$app->mode}.conf
+  #in etc/ado.$mode.conf
   plugins =>[
     #...
-    {name => 'auth', config => {
-        auth_methods =>['ado', 'facebook',...],
-        routes => [...]
-      }
-    }
+    'auth',
     #...
-  ]
+  ],
 
 =head1 DESCRIPTION
 
 L<Ado::Plugin::Auth> is a plugin that authenticates users to an L<Ado> system.
-Users can be authenticated locally or using (TODO!) Facebook, Google, Twitter
-and other authentication service-providers.
+Users can be authenticated locally or using (TODO!) Facebook, Github, Twitter
+and other authentication service-providers. 
 
-=head1 OPTIONS
+Note that the user's pasword is never sent over the network. When using the local
+authentication method (ado) a digest is prepared in the browser using JavaScript.
+The digest is sent and compared on the server side. The digest is different in
+every POST request. The other authentication methods use the services provided by
+well known service providers like Google, Facebook, Github etc. To use external
+authentication providers the module L<Mojolicious::Plugin::OAuth2> needs to be
+installed.
 
-The following options can be set in C<etc/ado.conf>.
+=head1 CONFIGURATION
+
+The following options can be set in C<etc/plugins/auth.$mode.conf>.
 You can find default options in C<etc/plugins/auth.conf>.
 
 =head2 auth_methods
@@ -183,15 +324,34 @@ authenticate a user. The services will be listed in the specified order
 in the partial template C<authbar.html.ep> that can be included
 in any other template on your site.
 
-  #in ado.${\$app->mode}.conf
-  plugins =>[
-    #...
-    {name => 'auth', config => {
-        auth_methods =>['ado', 'facebook',...]
-      }
-    }
-    #...
-  ]
+  #in etc/plugins/auth.$mode.conf
+  {
+    #methods which will be displayed in the "Sign in" menu
+    auth_methods => ['ado', 'google'],
+  }
+
+=head2 providers
+
+A Hash reference with keys representing names of providers (same as auth_methods)
+and values, containing the configurations for the specific providers.
+This option will be merged with already defined providers by
+L<Mojolicious::Plugin::OAuth2>.
+Add the rest of the needed configuration options to auth.development.conf or
+auth.production.conf only because this is highly sensitive and application
+specific information.
+
+  #Example for google:
+  google =>{
+      #client_id
+      key =>'123456654321abcd.apps.googleusercontent.com',
+      secret =>'Y0uRS3cretHEre',
+      scope=>'profile email',
+      info_url => 'https://www.googleapis.com/userinfo/v2/me',
+      },
+
+=head2 routes
+
+Currently defined routes are described in L</ROUTES>.
 
 =head1 CONDITIONS
 
@@ -201,17 +361,23 @@ To find more about conditions read L<Mojolicious::Guides::Routing/Conditions>.
 =head2 authenticated
 
 Condition for routes used to check if a user is authenticated.
-Additional parameters can be passed to specify the preferred 
-authentication method to be preselected in the login form
-if condition redirects to C</login/:auth_method>.
+
+=cut
+
+#TODO:?
+#Additional parameters can be passed to specify the preferred
+#authentication method to be preselected in the login form
+#if condition redirects to C</login/:auth_method>.
+
+=pod
 
   # add the condition programatically
   $app->routes->route('/ado-users/:action', over => {authenticated=>1});
   $app->routes->route('/ado-users/:action', 
-    over => [authenticated => 1, authz => {group => 'admin'}]
+    over => [authenticated => 1, ingroup => 'admin']
   );
 
-  #in ado.conf or ado.${\$app->mode}.conf
+  #in etc/ado.$mode.conf or etc/plugins/foo.$mode.conf
   routes => [
     #...
     {
@@ -220,7 +386,7 @@ if condition redirects to C</login/:auth_method>.
       
       # only authenticated users can edit and delete users,
       # and only if they are authorized to do so
-      over => [authenticated => 1, authz => {group => 'admin'}],
+      over => [authenticated => 1, ingroup => 'admin'],
       to =>'ado-users#edit'
     }
   ],
@@ -229,12 +395,12 @@ if condition redirects to C</login/:auth_method>.
 
 Checks if a user is in the given group. Returns true or false
   
-  # in etc/routes.conf or etc/fooplugin.conf
+  # in etc/plugins/routes.conf or etc/plugins/foo.conf
   {
     route => '/vest', 
     via => ['GET'], 
     to => 'vest#screen', 
-    over => {authenticated => 1, ingroup => 'foo'},
+    over => [authenticated => 1, ingroup => 'foo'],
   }
   # programatically
   $app->routes->route('/ado-users/:action', over => {ingroup => 'foo'});
@@ -244,18 +410,55 @@ Checks if a user is in the given group. Returns true or false
 L<Ado::Plugin::Auth> provides the following helpers for use in  
 L<Ado::Control> methods and templates.
 
+=head2 login_ado
 
+Finds and logs in a user locally. Returns true on success, false otherwise.
+
+=head2 login_google
+
+Called via C</login/google>. Finds an existing user and logs it in via Google.
+Creates a new user if it does not exist and logs it in via Google.
+The new user can login only via Google.
+Returns true on success, false otherwise.
+
+=head1 HOOKS
+
+Ado::Plugin::Auth emits the following hooks.
+
+=head2 after_user_add
+
+  $app->hook(after_build_tx => sub {
+    my ($c, $user, $raw_data) = @_;
+    my $group = $user->add_to_group(ingroup=>'vest');
+    ...
+  });
+
+In your plugin you can define some functionality to be executed right after a user
+is added. For example add a user to a group after registration. Passed the controller,
+the newly created C<$user> and the $raw_data used to create the user.
 
 =head1 ROUTES
 
 L<Ado::Plugin::Auth> provides the following routes (actions):
 
+=head2 /authorize/:auth_method
+
+Redirects to an OAuth2 provider consent screen where the user can authorize L<Ado>
+to use his information or not.
+Currently L<Ado> supports only Google.
+
 =head2 /login
 
-  /login/:auth_method
+  /login/ado
 
 If accessed using a C<GET> request displays a login form.
-If accessed via C<POST> performs authentication using C<:auth_method>.
+If accessed via C<POST> performs authentication using C<ado> system database.
+
+  /login/google
+
+Google consent screen redirects to this action. 
+This action is handled by L</login_google>.
+
 
 =head2 /logout
 
@@ -265,7 +468,8 @@ Expires the session and redirects to the base URL.
 
 =head1 TEMPLATES
 
-L<Ado::Plugin::Auth> embeds the following templates. You can run C<ado inflate> and modify them.
+L<Ado::Plugin::Auth> embeds the following templates.
+You can run C<ado inflate> and modify the inflated files.
 Usage examples can be found at L<http://localhost:3000> after starting ado.
 
 =head2 partials/authbar.html.ep
@@ -292,24 +496,22 @@ L<Ado::Plugin> and implements the following new ones.
 This method is called by C<$app-E<gt>plugin>.
 Registers the plugin in L<Ado> application and merges authentication 
 configuration from C<$MOJO_HOME/etc/ado.conf> with settings defined in
-C<$MOJO_HOME/etc/plugins/auth.conf>. Authentication settings defined in C<ado.conf>
-will overwrite those defined in C<plugins/auth.conf>.
+C<$MOJO_HOME/etc/plugins/auth.conf>. Authentication settings defined in
+C<plugins/auth.$mode.conf> will override those defined in C<plugins/auth.conf>.
+Authentication settings defined in C<ado.conf> will override both.
 
 =head1 TODO
 
 The following authentication methods are in the TODO list:
-facebook, linkedin, google.
+facebook, linkedin, github.
 Others may be added later.
 
 =head1 SEE ALSO
 
+L<Mojolicious::Plugin::OAuth2>,
 L<Ado::Plugin>, L<Ado::Manual::Plugins>, L<Mojolicious::Plugins>, 
 L<Mojolicious::Plugin>, L<Mojolicious::Guides::Routing/Conditions>
 
-=head1 SPONSORS
-
-The original author
-
 =head1 AUTHOR
 
 Красимир Беров (Krasimir Berov)
@@ -333,14 +535,17 @@ __DATA__
 
 @@ partials/authbar.html.ep
 %# displayed as a menu item
+% my $providers = config('Ado::Plugin::Auth')->{providers};
 <div class="right compact menu" id="authbar">
 % if (user->login_name eq 'guest') {
   <div class="ui simple dropdown item">
     <i class="sign in icon"></i><%=l('Sign in') %>
     <div class="menu">
-    % for my $auth(@{app->config('auth_methods')}){
-      <a href="<%=url_for("login/$auth")->to_abs %>" 
-        title="<%=ucfirst l($auth) %>" class="item">
+    % my $action ='login';
+    % for my $auth (@{app->config('auth_methods')}) {
+      % if ($auth ne 'ado') {$action ='authorize';}
+      <a href="<%=url_for("/$action/$auth")->to_abs %>" 
+        title="<%=ucfirst l($auth) %>" class="item <%= $auth %>">
         <i class="<%=$auth %> icon"></i>
       </a>
     % }    
@@ -370,7 +575,7 @@ __DATA__
       <%= stash->{error_login} %></div>
     % }
     <div class="field auth_methods">
-      % for my $auth(@{app->config('auth_methods')}){
+      % for my $auth(@{config('auth_methods')}){
       <span class="ui toggle radio checkbox">
         <input name="_method" type="radio" id="<%=$auth %>_radio"
           %== (stash->{auth_method}//'') eq $auth ? 'checked="checked"' : ''
